=== modified file '__init__.py'
--- __init__.py	2007-04-10 14:14:25 +0000
+++ __init__.py	2007-04-10 16:24:05 +0000
@@ -1,12 +1,72 @@
+from difflib import SequenceMatcher
+
 class MultiParent(object):
 
-    def __init__(self):
-        self.hunks = []
+    def __init__(self, hunks=None):
+        if hunks is not None:
+            self.hunks = hunks
+        else:
+            self.hunks = []
+
+    def __repr__(self):
+        return "MultiParent(%r)" % self.hunks
+
+    def __eq__(self, other):
+        if self.__class__ is not other.__class__:
+            return False
+        return (self.hunks == other.hunks)
 
     @staticmethod
     def from_lines(text, parents=()):
-        diff = MultiParent()
-        diff.hunks.append(NewText(text))
+        def compare(parent):
+            return SequenceMatcher(None, parent, text).get_matching_blocks()
+        parent_comparisons = [compare(p) for p in parents]
+        cur_line = 0
+        new_text = NewText([])
+        parent_text = []
+        block_iter = [iter(i) for i in parent_comparisons]
+        diff = MultiParent([])
+        def next_block(p):
+            try:
+                return block_iter[p].next()
+            except StopIteration:
+                return None
+        cur_block = [next_block(p) for p, i in enumerate(block_iter)]
+        while cur_line < len(text):
+            #import pdb; pdb.set_trace()
+            best_match = None
+            for p, block in enumerate(cur_block):
+                if block is None:
+                    continue
+                i, j, n = block
+                while j + n < cur_line:
+                    block = cur_block[p] = next_block(p)
+                    if block is None:
+                        break
+                    i, j, n = block
+                if block is None:
+                    continue
+                if j > cur_line:
+                    continue
+                offset = cur_line - j
+                i += offset
+                j = cur_line
+                n -= offset
+                if n == 0:
+                    continue
+                if best_match is None or n > best_match.num_lines:
+                    best_match = ParentText(p, i, j, n)
+            if best_match is None:
+                new_text.lines.append(text[cur_line])
+                cur_line += 1
+            else:
+                if len(new_text.lines) > 0:
+                    diff.hunks.append(new_text)
+                    new_text = NewText([])
+                diff.hunks.append(best_match)
+                cur_line += best_match.num_lines
+        if len(new_text.lines) > 0:
+            diff.hunks.append(new_text)
         return diff
 
     @classmethod
@@ -24,3 +84,24 @@
         if self.__class__ is not other.__class__:
             return False
         return (other.lines == self.lines)
+
+    def __repr__(self):
+        return 'NewText(%r)' % self.lines
+
+
+class ParentText(object):
+
+    def __init__(self, parent, parent_pos, child_pos, num_lines):
+        self.parent = parent
+        self.parent_pos = parent_pos
+        self.child_pos = child_pos
+        self.num_lines = num_lines
+
+    def __repr__(self):
+        return 'ParentText(%(parent)r, %(parent_pos)r, %(child_pos)r,'\
+            ' %(num_lines)r)' % self.__dict__
+
+    def __eq__(self, other):
+        if self.__class__ != other.__class__:
+            return False
+        return (self.__dict__ == other.__dict__)

=== modified file 'tests/test_multiparent.py'
--- tests/test_multiparent.py	2007-04-10 14:14:25 +0000
+++ tests/test_multiparent.py	2007-04-10 16:24:05 +0000
@@ -3,12 +3,41 @@
 import multiparent
 
 LINES_1 = "a\nb\nc\nd\ne\n".splitlines(True)
+LINES_2 = "a\nc\nd\ne\n".splitlines(True)
+
+
+class Mock(object):
+
+    def __init__(self, **kwargs):
+        self.__dict__ = kwargs
+
 
 class TestMulti(TestCase):
 
-    def test_compare(self):
-        diff = multiparent.MultiParent.from_lines(LINES_1)
-        self.assertEqual(diff.hunks, [multiparent.NewText(LINES_1)])
+    def test_compare_no_parent(self):
+        diff = multiparent.MultiParent.from_lines(LINES_1)
+        self.assertEqual([multiparent.NewText(LINES_1)], diff.hunks)
+
+    def test_compare_one_parent(self):
+        diff = multiparent.MultiParent.from_lines(LINES_1, [LINES_2])
+        self.assertEqual([multiparent.ParentText(0, 0, 0, 1),
+                          multiparent.NewText(['b\n']),
+                          multiparent.ParentText(0, 1, 2, 3)],
+                         diff.hunks)
+
+    def test_eq(self):
+        diff = multiparent.MultiParent.from_lines(LINES_1)
+        diff2 = multiparent.MultiParent.from_lines(LINES_1)
+        self.assertEqual(diff, diff2)
+        diff3 = multiparent.MultiParent.from_lines(LINES_2)
+        self.assertFalse(diff == diff3)
+        self.assertFalse(diff == Mock(hunks=[multiparent.NewText(LINES_1)]))
+        self.assertEqual(multiparent.MultiParent(
+                         [multiparent.NewText(LINES_1),
+                          multiparent.ParentText(0, 1, 2, 3)]),
+                         multiparent.MultiParent(
+                         [multiparent.NewText(LINES_1),
+                          multiparent.ParentText(0, 1, 2, 3)]))
 
 
 class TestNewText(TestCase):
@@ -17,7 +46,17 @@
         self.assertEqual(multiparent.NewText([]), multiparent.NewText([]))
         self.assertFalse(multiparent.NewText(['a']) ==
                          multiparent.NewText(['b']))
-        class ThingWithLines(object):
-            def __init__(self):
-                self.lines = ['a']
-        self.assertFalse(multiparent.NewText(['a']) == ThingWithLines())
+        self.assertFalse(multiparent.NewText(['a']) == Mock(lines=['a']))
+
+
+class TestParentText(TestCase):
+
+    def test_eq(self):
+        self.assertEqual(multiparent.ParentText(1, 2, 3, 4),
+                         multiparent.ParentText(1, 2, 3, 4))
+        self.assertFalse(multiparent.ParentText(1, 2, 3, 4) ==
+                         multiparent.ParentText(2, 2, 3, 4))
+
+        self.assertFalse(multiparent.ParentText(1, 2, 3, 4) ==
+                         Mock(parent=1, parent_pos=2, child_pos=3,
+                              num_lines=4))

